import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import Config from './config.js';
class Utils {
    static init(service, extension, metadata, settings) {
        Utils.extension = extension;
        Utils.metadata = metadata;
        Utils.settings = settings;
        Config.settings = settings;
        Utils.debugMode = Config.get_boolean('debug-mode');
        if (service === 'extension') {
            Utils.settings.connect('changed::debug-mode', (sender, key) => {
                this.debugMode = sender.get_boolean(key);
                if (this.debugMode) {
                    this.clean_logFile();
                }
                else {
                    this.delete_logFile();
                }
            });
            if (this.debugMode) {
                this.clean_logFile();
            }
        }
    }
    static delete_logFile() {
        try {
            const log = Utils.getLogFile();
            if (log) {
                if (log.query_exists(null))
                    log.delete(null);
            }
        }
        catch (e) {
            console.error(e);
        }
    }
    static clean_logFile() {
        try {
            const log = Utils.getLogFile();
            if (log) {
                if (log.query_exists(null))
                    log.delete(null);
                log.create_readwrite(Gio.FileCreateFlags.REPLACE_DESTINATION, null);
            }
        }
        catch (e) {
            console.error(e);
        }
    }
    static fromStyles(color) {
        return {
            red: color.red / 255,
            green: color.green / 255,
            blue: color.blue / 255,
            alpha: color.alpha / 255,
        };
    }
    static getLogFile() {
        try {
            const dataDir = GLib.get_user_cache_dir();
            const destination = GLib.build_filenamev([dataDir, 'vuemeter-system', 'debug.log']);
            const destinationFile = Gio.File.new_for_path(destination);
            if (destinationFile &&
                GLib.mkdir_with_parents(destinationFile.get_parent().get_path(), 0o755) === 0)
                return destinationFile;
        }
        catch (e) {
            console.error(e);
        }
        return null;
    }
    static logToFile(header, message) {
        if (this.debugMode) {
            const log = Utils.getLogFile();
            if (log) {
                try {
                    const date = new Date();
                    const time = date.toISOString().split('T')[1].slice(0, -1);
                    const outputStream = log.append_to(Gio.FileCreateFlags.NONE, null);
                    const buffer = new TextEncoder().encode(`${time}: ${header} - ${message}\n`);
                    outputStream.write_all(buffer, null);
                }
                catch (e) {
                    console.error(Utils.HEADER, e);
                }
            }
        }
    }
    static concat(...data) {
        let message = '';
        for (const str of data) {
            if (message.length === 0)
                message += str;
            else
                message += ' ' + str;
        }
        return message;
    }
    static log(...data) {
        const message = Utils.concat(...data);
        console.log(Utils.HEADER, message);
        Utils.logToFile('INFO', message);
    }
    static debug(...data) {
        const message = Utils.concat(...data);
        console.log(Utils.HEADER, message);
        Utils.logToFile('DEBUG', message);
    }
    static error(...data) {
        const message = Utils.concat(...data);
        console.log(Utils.HEADER, message);
        Utils.logToFile('ERROR', message);
    }
    static roundFloatingPointNumber(num) {
        const numStr = num.toString();
        const decimalIndex = numStr.indexOf('.');
        if (decimalIndex === -1)
            return num;
        const fractionLength = numStr.length - decimalIndex - 1;
        let precision = Math.min(10, fractionLength);
        if (fractionLength > 10)
            precision = fractionLength - 10;
        return Number(num.toFixed(precision));
    }
    static lookupColor(widget, name, defaultColor) {
        if (widget.get_stage()) {
            let cachedColorPerWidget = this.cachedColor[widget.name];
            if (cachedColorPerWidget === undefined) {
                cachedColorPerWidget = {};
                this.cachedColor[widget.name] = cachedColorPerWidget;
            }
            if (cachedColorPerWidget[name]) {
                defaultColor = cachedColorPerWidget[name];
            }
            else {
                const themeNode = widget.get_theme_node();
                const [hasColor, color] = themeNode.lookup_color(name, true);
                if (hasColor) {
                    Utils.debug(`${widget.name}::lookupColor->${this.name}, name:${name} red: ${color.red}, blue: ${color.blue}, green: ${color.green}, alpha:${color.alpha}`);
                    defaultColor = {
                        red: color.red / 255.0,
                        blue: color.blue / 255.0,
                        green: color.green / 255.0,
                        alpha: color.alpha / 255.0,
                    };
                    cachedColorPerWidget[name] = defaultColor;
                }
                else {
                    Utils.debug(`${widget.name}::lookupColor->${this.name}, name:${name} not found, use red: ${defaultColor.red}, blue: ${defaultColor.blue}, green: ${defaultColor.green}, alpha:${defaultColor.alpha}`);
                }
            }
        }
        return defaultColor;
    }
}
Utils.HEADER = 'vuemeter-system';
Utils.debugMode = false;
Utils.cachedColor = {};
export default Utils;
